﻿'use strict'

var printbot = require('./modules/printbot-connector');
var octoprint = require('./modules/octoprint');
var helpers = require('./modules/helpers');

function PSI(pbUrl, pbAuthToken) {
    this.PB = new printbot(pbUrl, pbAuthToken);
    //this.OP = []; // SET IN THE printbotConnect method
    this.Helpers = new helpers();
}

PSI.prototype.handleError(errData, VoiceAss) { // I will call them voice asses from now on.
    /* var errorModel = {
        statusCode: response.statusCode,
        body: body,
        error: error,
        errMsg: "Failed to connect to PrintBot."
    }*/

    console.log("EXITED WITH ERROR => ", errData);

    if (errData.errMsg) {
        Alexa.say(errData.errMsg);
    }
    else if (errData.statusCode) {
        switch (errData.statusCode) {
            case ERRCODE["NoOctoprintInstance"]:
                GA.tell("I couln't find a OctoPrint connected to your profile. Go to the Printer Squid app and add your instance there.");
                break;
            case ERRCODE["InvalidOctoprintDetails"]:
                VoiceAss.tell(
                    "Your OctoPrint URL or API key seems to be missing.\n" +
                    "Go to the PrintBot app and make sure you have entered both your URL and API key.");
                break;
            case ERRCODE["FailedNonRunningCheck"]:
                VoiceAss.tell("You can't do that while you're printing.");
                break;
            case ERRCODE["StartJob_AlreadyPrinting"]:
                VoiceAss.tell("Your printer is already printing.");
                break;
            case ERRCODE["JobActions_NothingPrinting"]:
                VoiceAss.tell("You are not printing anything right now.");
                break;
            case ERRCODE["WebAppUnauthorized"]:
                VoiceAss.tell("Please go to your Google Assistant app to set up your PrintBot account.");
                break;
        }
    }
    else {
        Alexa.say("Something went wrong.");
    }
}

function printbotConnect(PSI, onError, onSuccess) {
    console.log("PSI_ENTERED_printbotConnect");

    // Verifies user and if all user details are present
    PSI.PB.verifyUserAndGetUserData(PSI, onError,
        (success) => {
            console.log("PRINTBOT_CONNECTED");

            if (success) {
                var userData = PSI.PB.USER_DATA;

                if (!userData.activeInstanceId) {
                    onError({
                        statusCode: 102,
                        body: userData,
                    });
                }
                else if (!userData.octoprintAPIKey || !userData.octoprintUrl) {
                    onError({
                        statusCode: 103,
                        body: userData,
                    });
                }
                else {
                    PSI.OP = new octoprint(PSI.PB.USER_DATA.octoprintUrl, PSI.PB.USER_DATA.octoprintAPIKey);

                    console.log("PRINTBOT_INITIALIZED");

                    onSuccess();
                }
            }
            else {
                console.log("PRINTBOT_FAILED");
                onError(PSI.OP.RESPONSE);
            }
        }
    );
}

function ensureNonRunningPrinter(PSI, onError, onSuccess) {
    console.log("PSI_ENTERED_ensurePrinterIsNotPrinting");

    PSI.OP.checkIfPrinterIsPrinting(onError, (isPrinting) => {
        if (isPrinting) {
            onError({
                statusCode: 110
            });
        }
        else {
            onSuccess();
        }
    });
};

// INTENT ACTIONS

PSI.prototype.getOctoprintVersion = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.getOctoprintVersion(onError, (data) => {
            onSuccess(data);
        });
    });
};

PSI.prototype.getCurrentProgress = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.getCurrentJobState(onError, (data) => {
            var p = Math.round(data.progress.completion);
            var tw = GetTimeWithWords(data.progress.printTimeLeft);

            var userFriendlyData = {
                progress: p,
                timeWithWords: tw,
            }

            onSuccess(userFriendlyData);
        });
    });

};

PSI.prototype.getCurrentTemperatures = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.getCurrentTemperatures(onError, (data) => {
            onSuccess(data);
        });
    });
};

// Get the print status, will be used to prompt the user for the selected file name
PSI.prototype.prepareForStartPrint = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        ensureNonRunningPrinter(this, onError, () => {
            this.OP.getCurrentJobState(onError, (jobState) => {
                onSuccess(jobState.job.file.name);
            });
        });
    });
};

// Checks if the printer is printing, returns true/false
PSI.prototype.checkIfPrinterIsPrinting = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.checkIfPrinterIsPrinting(onError, (isPrinting) => {
            onSuccess(isPrinting);
        });
    });
};

// INTENT FUNCTIONS START HERE

// Tasks
PSI.prototype.autoHome = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        ensureNonRunningPrinter(this, onError, () => {
            this.OP.autoHome(onError, () => onSuccess());
        });
    });
};

PSI.prototype.disableSteppers = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        ensureNonRunningPrinter(this, onError, () => {
            this.OP.disableSteppers(onError, () => onSuccess());
        });
    });
};

PSI.prototype.cooldown = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        ensureNonRunningPrinter(this, onError, () => {
            this.OP.cooldown(onError, () => onSuccess());
        });
    });
};

// Print Job Commands
PSI.prototype.startPrint = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.startPrint(
            (error) => {
                if (error.statusCode == 409) { // 409 Conflict is returned when issuing start print while the printer is printing
                    onError({
                        statusCode: 111
                    });
                }
                else { // Otherwise just pass the error for standart handling
                    onError(error);
                }
            },
            () => {
                onSuccess();
            }
        );
    });
};

PSI.prototype.stopPrint = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.stopPrint(
            (error) => {
                if (error.statusCode == 409) { // 409 Conflict is returned when issuing cancel print while the printer is not printing
                    onError({
                        statusCode: 112
                    });
                }
                else { // Otherwise just pass the error for standart handling
                    onError(error);
                }
            },
            () => {
                onSuccess();
            }
        );
    });
};

PSI.prototype.pausePrint = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.pausePrint(
            (error) => {
                if (error.statusCode == 409) { // 409 Conflict is returned when issuing pause print while the printer is not printing
                    onError({
                        statusCode: 112
                    });
                }
                else { // Otherwise just pass the error for standart handling
                    onError(error);
                }
            },
            () => {
                onSuccess();
            }
        );
        this.OP.stopPrint(onError, () => onSuccess());
    });
};

PSI.prototype.resumePrint = function (onSuccess, onError) {
    printbotConnect(this, onError, () => {
        this.OP.resumePrint(
            (error) => {
                if (error.statusCode == 409) { // 409 Conflict is returned when issuing resume print while the printer is not printing
                    onError({
                        statusCode: 112
                    });
                }
                // Otherwise just pass the error for standart handling
                else {
                    onError(error);
                }
            },
            () => {
                onSuccess();
            }
        );
        this.OP.stopPrint(onError, () => onSuccess());
    });
};

// EXPORT
module.exports = PSI;

// HELPER METHODS
function octoprintRequestSettings(url, key, method, path, data) {
    var fullUrl = url + '/' + path;

    return {
        uri: fullUrl,
        method: method,
        headers: {
            'X-Api-Key': key
        },
        json: data
    };
}

function GetTimeWithWords(seconds) {
    if (!seconds) {
        return null;
    }

    var weeks = Math.floor(seconds / 604800);
    var days = Math.floor(seconds / 86400) % 7;
    var hours = Math.floor(seconds / 3600) % 24;
    var minutes = Math.floor(seconds / 60) % 60;
    var seconds = seconds % 60;

    var durations = [];

    if (weeks) {
        var plural = weeks > 1 ? 's' : '';
        durations.push(`${weeks} week${plural}`);
    }

    if (days) {
        var plural = days > 1 ? 's' : '';
        durations.push(`${days} day${plural}`);
    }

    if (hours) {
        var plural = hours > 1 ? 's' : '';
        durations.push(`${hours} hour${plural}`);
    }

    if (minutes) {
        var plural = minutes > 1 ? 's' : '';
        durations.push(`${minutes} minute${plural}`);
    }

    // if (seconds) {
    //     var plural = seconds > 1 ? 's' : '';
    //     durations.push(`${seconds} second${plural}`);
    // }

    if (durations.length == 1) {
        return durations[0];
    }
    else {
        var msg = "";

        for (var i = 0; i < durations.length; i++) {
            if (i == durations.length - 1) {
                msg += " and ";
            }
            else {
                msg += ", ";
            }

            msg += durations[i];
        }

        return msg;
    }
}