﻿'use strict'

var request = require('request');

function PB(printbotUrl, printbotAuthToken) {
    this.PRINTBOT_URL = printbotUrl;
    this.PRINTBOT_AUTH_TOKEN = printbotAuthToken;
}

PB.prototype.verifyUserAndGetUserData = function (PSI, onError, onSuccess) {
    var debug = true;

    var requestSettings = printBotRequestSettings(this.PRINTBOT_URL, this.PRINTBOT_AUTH_TOKEN);

    log(debug, () => { console.log("PRINTBOT_REQUEST => ", requestSettings); });

    request(requestSettings,
        function (error, response, body) {
            if (!error && response.statusCode >= 200 && response.statusCode < 300) {
                var userData = JSON.parse(body);
                log(debug, () => { console.log(`PRINTBOT_RESPONSE (${response.statusCode}) => `, userData); });

                PSI.PB.USER_DATA = userData;

                onSuccess(body);
            }
            else if (response.statusCode == 401) {
                log(debug, () => { console.log(`UNAUTHORIZED`); });
                log(debug, () => { console.log(`PRINTBOT_RESPONSE (${response.statusCode}) => `, body); });

                onError({
                    statusCode: response.statusCode,
                });
            }
            else {
                log(debug, () => { console.log(`PRINTBOT_ERR_MSG (${response.statusCode}) => `, error) });
                log(debug, () => { console.log(`PRINTBOT_ERR_BODY => `, body); });

                onError({
                    statusCode: response.statusCode,
                    body: body,
                    error: error,
                    errMsg: "Failed to connect to PrintBot."
                });
            }
        }
    );
};

// return (success, data, message)
module.exports = PB;

function printBotRequestSettings(url, authToken) {
    var fullUrl = url + '/api/getUserData';

    return {
        uri: fullUrl,
        method: 'GET',
        headers: {
            'Accept': 'application/json',
            'Authorization': `Bearer ${authToken}`
        }
    };
}

function log(shouldLog, cb) {
    if (shouldLog) {
        cb();
    }
}